package eu.greenlightning.gdx.labyrinthchallenge.labyrinth;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.Interpolation;

import eu.greenlightning.gdx.labyrinthchallenge.items.Item;

public class Labyrinth {

	private final int difficulty;
	private final int size;
	private final int height;

	private Material[][][] tiles;
	private Material[][][] horizontalWalls;
	private Material[][][] verticalWalls;
	private Item[][][] items;

	private int current = 0;
	private int next = 0;
	private Interpolation interpolation = Interpolation.linear;
	private float time = 0;
	private float total = 0.2f;
	private float alpha = 0;

	public Labyrinth(int difficulty) {
		this.difficulty = difficulty;
		this.size = 3 + 2 * difficulty;
		this.height = 3;
		init();
		fillLevel(0, Material.GRASS, Material.STONE);
		fillLevel(1, Material.DIRT, Material.STONE);
		fillLevel(2, Material.STONE, Material.GRASS);
	}

	private void init() {
		tiles = new Material[getSize()][getSize()][getHeight()];
		horizontalWalls = new Material[getSize()][getSize() + 1][getHeight()];
		verticalWalls = new Material[getSize() + 1][getSize()][getHeight()];
		items = new Item[getSize()][getSize()][getHeight()];
	}

	private void fillLevel(int level, Material ground, Material walls) {
		fillGround(level, ground);
		fillWalls(level, walls);
	}

	private void fillGround(int level, Material material) {
		for (int x = 0; x < getSize(); x++) {
			for (int y = 0; y < getSize(); y++) {
				tiles[x][y][level] = material;
			}
		}
	}

	private void fillWalls(int level, Material material) {
		for (int x = 0; x < getSize(); x++) {
			for (int y = 0; y < getSize() + 1; y++) {
				horizontalWalls[x][y][level] = material;
			}
		}
		for (int x = 0; x < getSize() + 1; x++) {
			for (int y = 0; y < getSize(); y++) {
				verticalWalls[x][y][level] = material;
			}
		}
	}

	public int getDifficulty() {
		return difficulty;
	}

	public int getSize() {
		return size;
	}

	public int getHeight() {
		return height;
	}

	public void clearTop(int x, int y, int z) {
		horizontalWalls[x][y + 1][z] = Material.EMPTY;
	}

	public void clearBottom(int x, int y, int z) {
		horizontalWalls[x][y][z] = Material.EMPTY;
	}

	public void clearRight(int x, int y, int z) {
		verticalWalls[x + 1][y][z] = Material.EMPTY;
	}

	public void clearLeft(int x, int y, int z) {
		verticalWalls[x][y][z] = Material.EMPTY;
	}

	public void interact(int x, int y) {
		if (hasItem(x, y)) {
			getItem(x, y).interact(this);
		}
	}

	public void setItem(Item item, int x, int y, int z) {
		items[x][y][z] = item;
	}

	public boolean hasItem(int x, int y) {
		return hasItem(x, y, current);
	}

	public boolean hasItem(int x, int y, int z) {
		return items[x][y][z] != null;
	}

	public Item getItem(int x, int y) {
		return getItem(x, y, current);
	}

	public Item getItem(int x, int y, int z) {
		return items[x][y][z];
	}

	public void clearItem(int x, int y, int z) {
		items[x][y][z] = null;
	}

	public boolean isTopClear(int x, int y) {
		return isTopClear(x, y, current) && isTopClear(x, y, next);
	}

	public boolean isTopClear(int x, int y, int z) {
		return horizontalWalls[x][y + 1][z] == Material.EMPTY;
	}

	public boolean isBottomClear(int x, int y) {
		return isBottomClear(x, y, current) && isBottomClear(x, y, next);
	}

	public boolean isBottomClear(int x, int y, int z) {
		return horizontalWalls[x][y][z] == Material.EMPTY;
	}

	public boolean isRightClear(int x, int y) {
		return isRightClear(x, y, current) && isRightClear(x, y, next);
	}

	public boolean isRightClear(int x, int y, int z) {
		return verticalWalls[x + 1][y][z] == Material.EMPTY;
	}

	public boolean isLeftClear(int x, int y) {
		return isLeftClear(x, y, current) && isLeftClear(x, y, next);
	}

	public boolean isLeftClear(int x, int y, int z) {
		return verticalWalls[x][y][z] == Material.EMPTY;
	}

	public void update() {
		if (isChanging()) {
			time += Gdx.graphics.getDeltaTime();
			if (time >= total) {
				current = next;
			} else {
				alpha = interpolation.apply(time / total);
			}
		}
	}

	private boolean isChanging() {
		return next != current;
	}

	public void setLevel(int level) {
		next = current = level;
	}

	public void goUp() {
		next = current - 1;
		change();
	}

	public void goDown() {
		next = current + 1;
		change();
	}

	private void change() {
		time = 0;
		alpha = 0;
	}

	public void draw(Batch batch) {
		draw(batch, current, 1);
		if (isChanging()) {
			draw(batch, next, alpha);
		}
	}

	public void draw(Batch batch, int level, float alpha) {
		batch.setColor(new Color(1, 1, 1, alpha));
		drawTiles(batch, level);
		drawHorizontalWalls(batch, level);
		drawVerticalWalls(batch, level);
		drawItems(batch, level);
	}

	private void drawTiles(Batch batch, int level) {
		for (int x = 0; x < tiles.length; x++) {
			for (int y = 0; y < tiles[x].length; y++) {
				tiles[x][y][level].drawTile(batch, x * Material.SIZE, y * Material.SIZE);
			}
		}
	}

	private void drawHorizontalWalls(Batch batch, int level) {
		for (int x = 0; x < horizontalWalls.length; x++) {
			for (int y = 0; y < horizontalWalls[x].length; y++) {
				horizontalWalls[x][y][level].drawHorizontalWall(batch, x * Material.SIZE, y * Material.SIZE);
			}
		}
	}

	private void drawVerticalWalls(Batch batch, int level) {
		for (int x = 0; x < verticalWalls.length; x++) {
			for (int y = 0; y < verticalWalls[x].length; y++) {
				verticalWalls[x][y][level].drawVerticalWall(batch, x * Material.SIZE, y * Material.SIZE);
			}
		}
	}

	private void drawItems(Batch batch, int level) {
		for (int x = 0; x < getSize(); x++) {
			for (int y = 0; y < getSize(); y++) {
				if (hasItem(x, y, level)) {
					items[x][y][level].draw(batch, x, y);
				}
			}
		}
	}

	public void dispose() {
		for (int x = 0; x < getSize(); x++) {
			for (int y = 0; y < getSize(); y++) {
				for (int z = 0; z < getHeight(); z++) {
					if (hasItem(x, y, z)) {
						items[x][y][z].dispose();
					}
				}
			}
		}
	}

}
