package eu.greenlightning.gdx.labyrinthchallenge.labyrinth;

import java.util.ArrayList;
import java.util.List;

import com.badlogic.gdx.math.MathUtils;

import eu.greenlightning.gdx.labyrinthchallenge.items.Teleporter;
import eu.greenlightning.gdx.labyrinthchallenge.items.Treasure;

public class Runner {

	private LabyrinthGenerator generator;
	private Labyrinth labyrinth;
	private boolean master;
	private int length;
	private int target;
	private List<Position> visited;
	private List<Position> path;

	public Runner(LabyrinthGenerator generator, boolean master, int target, Position position) {
		this.generator = generator;
		this.labyrinth = generator.getLabyrinth();
		this.master = master;
		this.length = 0;
		this.target = target;
		visited = new ArrayList<Position>();
		path = new ArrayList<Position>();
		path.add(position);
		generator.visit(position);
		move();
		spawn();
		run();
	}

	private void move() {
		while (length < target) {
			if (!step()) {
				if (master) {
					backtrack();
				} else {
					break;
				}
			}
		}
	}

	private boolean step() {
		switch (MathUtils.random(0, 3)) {
			case 0:
				Position right = getPosition().getRight();
				if (isGood(right)) {
					moveRight();
					return true;
				} else {
					return tryOthers(right);
				}
			case 1:
				Position left = getPosition().getLeft();
				if (isGood(left)) {
					moveLeft();
					return true;
				} else {
					return tryOthers(left);
				}
			case 2:
				Position top = getPosition().getTop();
				if (isGood(top)) {
					moveUp();
					return true;
				} else {
					return tryOthers(top);
				}
			case 3:
				Position bottom = getPosition().getBottom();
				if (isGood(bottom)) {
					moveDown();
					return true;
				} else {
					return tryOthers(bottom);
				}
		}
		return false;
	}

	private boolean tryOthers(Position position) {
		if (isGood(getPosition().getHigher()) && isClear(getPosition()) && MathUtils.randomBoolean(0.2f)) {
			moveHigher();
			return true;
		}
		if (isGood(getPosition().getLower()) && isClear(getPosition()) && MathUtils.randomBoolean(0.2f)) {
			moveLower();
			return true;
		}
		Position right = getPosition().getRight();
		if (!right.equals(position) && isGood(right)) {
			moveRight();
			return true;
		}
		Position left = getPosition().getLeft();
		if (!left.equals(position) && isGood(left)) {
			moveLeft();
			return true;
		}
		Position top = getPosition().getTop();
		if (!top.equals(position) && isGood(top)) {
			moveUp();
			return true;
		}
		Position bottom = getPosition().getBottom();
		if (!bottom.equals(position) && isGood(bottom)) {
			moveDown();
			return true;
		}
		return false;
	}

	private boolean isGood(Position position) {
		return position.valid(labyrinth) && !generator.isVisited(position);
	}

	private boolean isClear(Position position) {
		return !labyrinth.hasItem(position.getX(), position.getY(), position.getZ())
			&& (!master || length + 1 < target);
	}

	private void moveHigher() {
		int rotation = MathUtils.random(3);
		labyrinth.setItem(new Teleporter(true, rotation), getX(), getY(), getZ());
		labyrinth.setItem(new Teleporter(false, rotation), getX(), getY(), getZ() - 1);
		path.add(getPosition().getHigher());
		visited.add(getPosition());
		generator.visit(getPosition());
		length++;
	}

	private void moveLower() {
		int rotation = MathUtils.random(3);
		labyrinth.setItem(new Teleporter(false, rotation), getX(), getY(), getZ());
		labyrinth.setItem(new Teleporter(true, rotation), getX(), getY(), getZ() + 1);
		path.add(getPosition().getLower());
		visited.add(getPosition());
		generator.visit(getPosition());
		length++;
	}

	private void moveRight() {
		labyrinth.clearRight(getX(), getY(), getZ());
		path.add(getPosition().getRight());
		visited.add(getPosition());
		generator.visit(getPosition());
		length++;
	}

	private void moveLeft() {
		labyrinth.clearLeft(getX(), getY(), getZ());
		path.add(getPosition().getLeft());
		visited.add(getPosition());
		generator.visit(getPosition());
		length++;
	}

	private void moveUp() {
		labyrinth.clearTop(getX(), getY(), getZ());
		path.add(getPosition().getTop());
		visited.add(getPosition());
		generator.visit(getPosition());
		length++;
	}

	private void moveDown() {
		labyrinth.clearBottom(getX(), getY(), getZ());
		path.add(getPosition().getBottom());
		visited.add(getPosition());
		generator.visit(getPosition());
		length++;
	}

	private void backtrack() {
		path.remove(path.size() - 1);
		length--;
		if (length == 0) {
			length = Integer.MAX_VALUE;
		}
	}

	private void spawn() {
		if (master) {
			labyrinth.setItem(new Treasure(), getX(), getY(), getZ());
		}
	}

	private void run() {
		for (Position position : visited) {
			new Runner(generator, false, MathUtils.random(5, 15), position);
		}
	}

	private int getX() {
		return getPosition().getX();
	}

	private int getY() {
		return getPosition().getY();
	}

	private int getZ() {
		return getPosition().getZ();
	}

	private Position getPosition() {
		int last = path.size() - 1;
		return path.get(last);
	}

}
