package eu.greenlightning.gdx.labyrinthchallenge.player;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.*;
import com.badlogic.gdx.math.Interpolation;

import eu.greenlightning.gdx.labyrinthchallenge.labyrinth.Labyrinth;
import eu.greenlightning.gdx.labyrinthchallenge.labyrinth.Material;

public class Player {

	private Labyrinth labyrinth;
	private Sprite sprite;
	private Animation anim;
	private float time = 0.2f;
	private Interpolator<Integer> x;
	private Interpolator<Integer> y;
	private Interpolator<Float> rotation;

	public Player(Labyrinth labyrinth, int x, int y) {
		this.labyrinth = labyrinth;
		createGraphics();
		this.x = new Interpolator<Integer>(x, 0.2f, Interpolation.linear);
		this.y = new Interpolator<Integer>(y, 0.2f, Interpolation.linear);
		this.rotation = new Interpolator<Float>(90f, 0.1f, Interpolation.pow2);
	}

	private void createGraphics() {
		Texture texture = new Texture("graphics/player.png");
		TextureRegion[][] frames = TextureRegion.split(texture, 32, 32);
		anim = new Animation(0.2f / 4, new TextureRegion[] { frames[0][1], frames[0][0], frames[0][2],
			frames[0][0] });
		sprite = new Sprite(frames[0][0]);
	}

	public int getStadingOnX() {
		return x.isChanging() ? x.getOld() : x.getCurrent();
	}

	public int getStandingOnY() {
		return y.isChanging() ? y.getOld() : y.getCurrent();
	}

	public float getCenterX() {
		return sprite.getX() + sprite.getWidth() / 2;
	}

	public float getCenterY() {
		return sprite.getY() + sprite.getHeight() / 2;
	}

	public void update() {
		time += Gdx.graphics.getDeltaTime();
		if (goUp()) {
			if (labyrinth.isTopClear(x.getCurrent(), y.getCurrent())) {
				time = 0;
				y.interpolate(y.getCurrent() + 1);
			}
			rotation.interpolate(90f);
		}
		if (goDown()) {
			if (labyrinth.isBottomClear(x.getCurrent(), y.getCurrent())) {
				time = 0;
				y.interpolate(y.getCurrent() - 1);
			}
			if (rotation.getCurrent() == 0f) {
				rotation.interpolate(360f, 270f);
			} else {
				rotation.interpolate(270f);
			}
		}
		if (goRight()) {
			if (labyrinth.isRightClear(x.getCurrent(), y.getCurrent())) {
				time = 0;
				x.interpolate(x.getCurrent() + 1);
			}
			if (rotation.getCurrent() == 270f) {
				rotation.interpolate(-90f, 0f);
			} else {
				rotation.interpolate(0f);
			}
		}
		if (goLeft()) {
			if (labyrinth.isLeftClear(x.getCurrent(), y.getCurrent())) {
				time = 0;
				x.interpolate(x.getCurrent() - 1);
			}
			rotation.interpolate(180f);
		}
		if (interact()) {
			labyrinth.interact(x.getCurrent(), y.getCurrent());
		}
		sprite.setRegion(anim.getKeyFrame(time));
		x.update();
		y.update();
		rotation.update();
		sprite.setPosition(x.getValue() * Material.SIZE, y.getValue() * Material.SIZE);
		sprite.setRotation(rotation.getValue());
	}

	private boolean goUp() {
		boolean touched = Gdx.input.isTouched() && Gdx.input.getX() > Gdx.graphics.getWidth() * 0.25
			&& Gdx.input.getX() < Gdx.graphics.getWidth() * 0.75
			&& Gdx.input.getY() < Gdx.graphics.getHeight() * 0.25;
		boolean pressed = Gdx.input.isKeyPressed(Keys.UP);
		return (touched || pressed) && notChanging();
	}

	private boolean goDown() {
		boolean touched = Gdx.input.isTouched() && Gdx.input.getX() > Gdx.graphics.getWidth() * 0.25
			&& Gdx.input.getX() < Gdx.graphics.getWidth() * 0.75
			&& Gdx.input.getY() > Gdx.graphics.getHeight() * 0.75;
		boolean pressed = Gdx.input.isKeyPressed(Keys.DOWN);
		return (touched || pressed) && notChanging();
	}

	private boolean goRight() {
		boolean touched = Gdx.input.isTouched() && Gdx.input.getX() > Gdx.graphics.getWidth() * 0.75;
		boolean pressed = Gdx.input.isKeyPressed(Keys.RIGHT);
		return (touched || pressed) && notChanging();
	}

	private boolean goLeft() {
		boolean touched = Gdx.input.isTouched() && Gdx.input.getX() < Gdx.graphics.getWidth() * 0.25;
		boolean pressed = Gdx.input.isKeyPressed(Keys.LEFT);
		return (touched || pressed) && notChanging();
	}

	private boolean interact() {
		boolean touched = Gdx.input.justTouched() && Gdx.input.getX() > Gdx.graphics.getWidth() * 0.25
			&& Gdx.input.getX() < Gdx.graphics.getWidth() * 0.75
			&& Gdx.input.getY() > Gdx.graphics.getHeight() * 0.25
			&& Gdx.input.getY() < Gdx.graphics.getHeight() * 0.75;
		boolean pressed = Gdx.input.isKeyJustPressed(Keys.SPACE);
		return touched || pressed;
	}

	private boolean notChanging() {
		return !x.isChanging() && !y.isChanging();
	}

	public void draw(Batch batch) {
		sprite.draw(batch);
	}

	public void dispose() {
		sprite.getTexture().dispose();
	}

}
